@@ -1,24 +1,20 @@
 library(catnet)
 
-## load the alarm data sample and ground-thruth network
-data(alarm)
 data(alarmnet)
-alarmnet
+cnPlot(alarmnet)
 
-eval <- cnEvaluate(object=alarmnet, data=alarm, perturbations=NULL, maxParentSet=3, echo=FALSE)
-##eval <- cnSearchOrder(data=alarm, perturbations=NULL, maxParentSet=3, parentSizes=NULL,maxComplexity=cnComplexity(alarmnet), nodeOrder=cnOrder(alarmnet))
-eval
-##pdf("alarmeval.pdf")
-cnPlot(eval)
-##dev.off()
+alarm.data <- cnSamples(alarmnet, 10000)
+
+eval <- cnSearchOrder(data=alarm.data,perturbations=NULL, maxParentSet=4, parentSizes=NULL, maxComplexity=2*cnComplexity(alarmnet), nodeOrder=cnOrder(alarmnet), echo=TRUE)
 
 bstnet<- cnFind(eval, cnComplexity(alarmnet))
 bstnet
 cnCompare(alarmnet, bstnet)
 
+cnCompare(alarmnet, cnFindBIC(eval))
 
 cnSetSeed(123)
-sanets1 <- cnSearchSA(data = alarm, perturbations = NULL, maxParentSet = 2, 
+sanets1 <- cnSearchSA(data = alarm.data, perturbations = NULL, maxParentSet = 2, 
      maxComplexity = 600, parentsPool = NULL, fixedParents = NULL, 
      selectMode = "BIC", tempStart = 10, tempCoolFact = 1, tempCheckOrders = 100, 
      maxIter = 100, orderShuffles = 0, stopDiff = 1e-10, numThreads=4, priorSearch = NULL)
@@ -31,8 +27,8 @@ sanet1@meta <- "sanet1, SA stage I"
 sanet1
 cnPlot(sanet1, "sanet1")
 
-cnSetSeed(456)
-sanets <- cnSearchSA(data = alarm, perturbations = NULL, maxParentSet = 2, 
+cnSetSeed(123)
+sanets <- cnSearchSA(data = alarm.data, perturbations = NULL, maxParentSet = 3, 
      maxComplexity = 600, parentsPool = NULL, fixedParents = NULL, 
      selectMode = "BIC", tempStart = 1e-2, tempCoolFact = 0.9, 
      tempCheckOrders = 20, maxIter = 100, orderShuffles = 4, stopDiff = 1e-2, 
